/*
 * Access.java
 *
 * Created on August 25, 2005, 1:34 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.sql.*;
import java.util.Calendar;
import java.util.Date;
import wms.ss.data.SlotSignup;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import blackboard.db.*;

/**
 * This represents an access record, which associates a signup sheet framework 
 * with a bb user or a bb course (by the bb PK1). Access determines who can see 
 * the sheet to sign up on it.
 * @author cwarren
 */
public class Access {

    private int accessId;
    private java.util.Date dateCreated, dateUpdated;
    private int ownerUserPk1, frameworkId, allowedPk;
    private String accessCategory;
    
    /**
     * A constant that has the string for the ACCESS_CATEGORY to associate a framework with a user
     */
    public static String ACCESS_CATEGORY_USER = "USER";
    /**
     * A constant that has the string for the ACCESS_CATEGORY to associate a framework with a course
     */
    public static String ACCESS_CATEGORY_COURSE = "COURSE";
    /**
     * A constant that has the string for the ACCESS_CATEGORY to make the sheet open to the public - NOT CURRENTLY SUPPORTED!
     */
    public static String ACCESS_CATEGORY_ALL = "ALL";
    
    /** static methods */

    /**
     * Create a basic access record, with no attributes set.
     */
    public Access() {
    }

    /**
     * Create a access record with the four main fields (owner, framework, allowed PK, 
     * and category) set.
     * @param ownerPk1 The bb PK of the user that owns this access record
     * @param frameworkId The id of the framework to which access is being granted
     * @param allowedPk1 The bb PK of the course or user that is being granted access to the framework
     * @param accessCategory The category of the access (user or course) - one of ACCESS_CATEGORY_USER or ACCESS_CATEGORY_COURSE
     * @see #ACCESS_CATEGORY_USER
     * @see #ACCESS_CATEGORY_COURSE
     */
    public Access(int ownerPk1, int frameworkId, int allowedPk1, String accessCategory) {
        this.setOwnerUserPk1(ownerPk1); 
        this.setFrameworkId(frameworkId);
        this.setAllowedPk(allowedPk1);
        this.setAccessCategory(accessCategory);
    }

    /**
     * Create an access record from a result set that is generated by 
     * selecting all the fields from the DB table that supports Access 
     * records.
     * @param res The result of selecting all fields from the associated DB table.
     * @throws java.sql.SQLException 
     */
    public Access(ResultSet res) 
        throws SQLException
    {
        this.accessId = res.getInt("SS_ACCESS_ID");
        this.dateCreated = res.getTimestamp("DATE_CREATED");
        this.setDateUpdated(res.getTimestamp("DATE_UPDATED"));
        this.setOwnerUserPk1(res.getInt("OWNER_USER_PK1")); 
        this.setFrameworkId(res.getInt("SS_FRAMEWORK_ID"));
        this.setAllowedPk(res.getInt("ALLOWED_PK1"));
        this.setAccessCategory(res.getString("ACCESS_CATEGORY"));
    }

    /**
     * Save the current object to the DB by updating an existing access record
     */
    public void save() {
    }

    /**
     * Save the current object to the DB as a new access record
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void saveNew()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        int newId = SqlUtil.runSqlGetIntField("SELECT wms_ss_ids.nextval AS new_id FROM dual", "new_id");
        this.accessId = newId;
        Calendar cal = Calendar.getInstance();
        this.dateCreated = cal.getTime();
        this.dateUpdated = cal.getTime();
        String sqlText = 
                "INSERT INTO wms_ss_access ("+
                    " ss_access_id, date_created, date_updated, " +
                    " deleted, owner_user_pk1, ss_framework_id, " +
                    " allowed_pk1, access_category) " +
               " VALUES ("
                    +Integer.toString(this.accessId)+"," // ss_access_id
                    +"SYSDATE," // date_created
                    +"SYSDATE," // date_updated
                    +"'N'," // deleted
                    +Integer.toString(this.getOwnerUserPk1())+"," // owner_user_pk1
                    +Integer.toString(this.getFrameworkId())+"," //ss_framework_id
                    +Integer.toString(this.getAllowedPk())+"," // allowed_pk1
                    +"'"+this.getAccessCategory()+"'" // access_category
                +")";
        SqlUtil.runSql(sqlText);
    }

    /**
     * The the ID of this access object
     * @return The id of this access object
     */

    public int getAccessId() {
        return accessId;
    }

    /**
     * Gets the date and time this object was created
     * @return The date this ojbject was created
     */
    public java.util.Date getDateCreated() {
        return dateCreated;
    }

    /**
     * Get the date-time of the alst timethis object was updated/changed
     * @return The date-time this object was last updated/changed
     */
    public java.util.Date getDateUpdated() {
        return dateUpdated;
    }

    /**
     * Set the date-teim this object is changed
     * @param dateUpdated The date-time this object was last updated/changed
     */
    public void setDateUpdated(java.util.Date dateUpdated) {
        this.dateUpdated = dateUpdated;
    }

    /**
     * Get the bb PK of the user that owns this Access record (this is the same as the owner 
     * of the framework to which access has been granted)
     * @return A bb PK for a user
     */
    public int getOwnerUserPk1() {
        return ownerUserPk1;
    }

    /**
     * Set the bb PK for the owner of this object
     * @param ownerUserPk1 a bb PK for a user
     */
    public void setOwnerUserPk1(int ownerUserPk1) {
        this.ownerUserPk1 = ownerUserPk1;
    }

    /**
     * Gets the id of the framwork to which this grants access
     * @return a framework ID
     */
    public int getFrameworkId() {
        return frameworkId;
    }

    /**
     * Sets the framework (as specified by its ID) to which this grants access.
     * @param frameworkId A framework ID
     */
    public void setFrameworkId(int frameworkId) {
        this.frameworkId = frameworkId;
    }

    /**
     * Get the bb PK of the entity (user or course - which is determined by getAccessCategory) which is getting access
     * @return a bb PK for a user or course
     */
    public int getAllowedPk() {
        return allowedPk;
    }

    /**
     * Sets the PK of the user or course for which this grants access
     * @param allowedPk a bb PK of a user or course
     */
    public void setAllowedPk(int allowedPk) {
        this.allowedPk = allowedPk;
    }

    /**
     * Gets the category of this access record - indicates whether this access record grants access for a user or for a course.
     * @return The category/type of entity for which this grants access
     * @see #ACCESS_CATEGORY_USER
     * @see #ACCESS_CATEGORY_COURSE
     */
    public String getAccessCategory() {
        return accessCategory;
    }

    /**
     * Sets what kind of access record this is - for a user or a course
     * @param accessCategory either ACCESS_CATEGORY_USER or ACCESS_CATEGORY_COURSE
     * @see #ACCESS_CATEGORY_USER
     * @see #ACCESS_CATEGORY_COURSE
     */
    public void setAccessCategory(String accessCategory) {
        this.accessCategory = accessCategory;
    }
}
