/*
 * AccessList.java
 *
 * Created on August 25, 2005, 1:34 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.util.ArrayList;
import wms.ss.db.Connect;
import blackboard.db.*;
import blackboard.data.course.Course;
import blackboard.data.user.User;
import java.sql.*;

/**
 * This class manages lists of Access objects which are related to each other (e.g. 
 * by being for the same framework, course, or user). The constructor is private and 
 * so creating an AccessList is only done through the various loadByX static methods. It also 
 * includes a few methods that encapsulate common actions (e.g. finding out whether a 
 * given user has an access record in the list).
 * @author cwarren
 */
public class AccessList extends java.util.ArrayList {

    private static String dataSourceTable = "WMS_SS_ACCESS";

    /**
     * Create a an AccessList that contains all the Access objects for a specified Framework
     * @param frameworkId The ID of the framework to which the Access objects in the list grant access
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return an AccessList
     */
    public static AccessList loadAllAccessByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return AccessList.loadAnyAccessByAnyIdAndFrameworkId (Access.ACCESS_CATEGORY_ALL, 0, frameworkId);
    }
    
    /**
     * Create a an AccessList that contains all the Access objects that have an access category 
     * of Access.ACCESS_CATEGORY_COURSE for a specified Framework
     * @param frameworkId The ID of the framework to which the Access objects in the list grant access
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return an AccessList
     */
    public static AccessList loadCourseAccessByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return AccessList.loadAnyAccessByAnyIdAndFrameworkId (Access.ACCESS_CATEGORY_COURSE, 0, frameworkId);
    }

    /**
     * Create a an AccessList that contains all the Access objects that have an access category 
     * of Access.ACCESS_CATEGORY_USER for a specified Framework
     * @param frameworkId The ID of the framework to which the Access objects in the list grant access
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return an AccessList
     */
    public static AccessList loadUserAccessByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return AccessList.loadAnyAccessByAnyIdAndFrameworkId (Access.ACCESS_CATEGORY_USER, 0, frameworkId);
    }

    /* accessCategory:from Access.ACCESS_CATEGORY_* */
    /**
     * A more flexible creation method which lets you specify variouys attributes of the 
     * Access objects that will be in the list.
     * @param accessCategory The Access category of the object in the resulting list 
     * (Access.ACCESS_CATEGORY_COURSE or ACCESS_CATEGORY_USER). If blank then 
     * the category will not be a factor in whether or not an Access is in the 
     * list.
     * @param allowedPk The BB primary key value of the thing (course or user) for which access 
     * is being granted for all Access objects in the resulting list. 0 means all 
     * Access objects regardless of who or what they grant access for.
     * @param frameworkId The id of the framwork to which access is being granted by all Access objects 
     * in the resulting list. A 0 indicates all Access objects, regardless of which
     * framework they grant access to.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return an AccessList
     */
    public static AccessList loadAnyAccessByAnyIdAndFrameworkId (String accessCategory, int allowedPk, int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT * " +
                " FROM "+dataSourceTable+" "+
                " WHERE deleted != 'Y' ";
        if (frameworkId != 0)
        {
            sqlText = sqlText + "  AND ss_framework_id = "+Integer.toString(frameworkId);
        }
        if (allowedPk != 0)
        {
            sqlText = sqlText + "  AND allowed_pk1 = "+Integer.toString(allowedPk);
        }
        if (accessCategory.equalsIgnoreCase(Access.ACCESS_CATEGORY_COURSE) ||
                accessCategory.equalsIgnoreCase(Access.ACCESS_CATEGORY_USER))
        {
            sqlText = sqlText + " AND access_category = '"+accessCategory.toUpperCase()+"'";
        }
        sqlText = sqlText + " ORDER BY access_category, allowed_pk1";
        if (1==0)
        {
            throw new SQLException("query is "+sqlText);
        }
        return AccessList.createFromSql(sqlText);
    }

    /** private static methods */
    private static AccessList createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        AccessList al = new AccessList();
        while (res.next())
        {
            Access a = new Access(res);
            al.add(a);
        }
        sql.close();
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
        
        return al;
    }

    /** object methods */

    /** constructors */
    private AccessList() {
        super();
    }
    
    /**
     * A utility method to return the BB pk from an Access object at the specified index 
     * (0-based) in this list.
     * @param idx The index of the Access object in the list
     * @return The BB pk associated with the Access object at that index
     */
    public int getPkAt(int idx)
    {
        Access a = (Access)this.get(idx);
        return a.getAllowedPk();
    }
    
    /**
     * A utility methods that checks this list for any Access object that grants access 
     * for the specified user.
     * @param bbUser the BB pk of the user to check for
     * @return true if there's an Access object in this that grants access for the indicated user. false otherwise.
     */
    public boolean hasUser(User bbUser)
    {
        int userPk = DataUtil.getPkFromBbId(bbUser.getId());
        for (int i = 0; i < this.size(); i++)
        {
            Access a = (Access)(this.get(i));
            if ((userPk == a.getAllowedPk()) &&
                    (a.getAccessCategory().equals(Access.ACCESS_CATEGORY_USER)))
            {
                return true;
            }
        }
        return false;
    }
    
    /**
     * A utility methods that checks this list for any Access object that grants access 
     * for the specified course.
     * @param bbCourse the BB pk of the course to check
     * @return true if there's an Access object in this that grants access for the indicated course. false otherwise.
     */
    public boolean hasCourse(Course bbCourse)
    {
        int coursePk = DataUtil.getPkFromBbId(bbCourse.getId());
        for (int i = 0; i < this.size(); i++)
        {
            Access a = (Access)(this.get(i));
            if ((coursePk == a.getAllowedPk()) &&
                    (a.getAccessCategory().equals(Access.ACCESS_CATEGORY_COURSE)))
            {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Removes all access from this all Access objects that grant access for the specified course.
     * @param bbCourse A BB pk for a course
     */
    public void removeCourse(Course bbCourse)
    {
        int coursePk = DataUtil.getPkFromBbId(bbCourse.getId());
        for (int i = 0; i < this.size(); i++)
        {
            Access a = (Access)(this.get(i));
            if ((coursePk == a.getAllowedPk()) &&
                    (a.getAccessCategory().equals(Access.ACCESS_CATEGORY_COURSE)))
            {
                this.remove(i);
                return;
            }
        }
        return;
    }
}
