/*
 * AdminList.java
 *
 * Created on January 4, 2006, 3:33 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.util.ArrayList;
import wms.ss.db.Connect;
import blackboard.db.*;
import blackboard.data.course.Course;
import blackboard.data.user.User;
import java.sql.*;

/**
 * This class manages lists of Admin objects which are related to each other (e.g. 
 * by being for the same framework or user). The constructor is private and 
 * so creating an AdminList is only done through the various loadByX static methods. It also 
 * includes a few methods that encapsulate common actions (e.g. finding out whether a 
 * given user has an Admin record in the list).
 * @author cwarren
 */
public class AdminList extends java.util.ArrayList {
    
    private static String dataSourceTable = "WMS_SS_VW_ADMIN_ND";

    //-------------------------------------------------------------------------

    /**
     * Gets all Admin records for a given framework - i.e. everyone who can administer
     * a framework.
     * @param frameworkId The ID of a framework
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return An AdminList of Admin objects for the users that can administer that framework.
     */
    public static AdminList loadAllAdminByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT * " +
                " FROM "+dataSourceTable+" "+
                " WHERE deleted != 'Y' " +
                " AND ss_framework_id = "+Integer.toString(frameworkId) +
                " ORDER BY admin_user_pk";

        return AdminList.createFromSql(sqlText);
    }

    /**
     * Gets all Admin records for a given user - i.e. all the frameworks the user can
     * administer.
     * @param adminUserPk a BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return An AdminList of Admin objects for the frameworks that the users that can administer.
     */
    public static AdminList loadAllAdminByAdminUserPk (int adminUserPk)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT * " +
                " FROM "+dataSourceTable+" "+
                " WHERE deleted != 'Y' " +
                " AND admin_user_pk = "+Integer.toString(adminUserPk) +
                " ORDER BY ss_framework_id";

        return AdminList.createFromSql(sqlText);
    }

    /** private static methods */
    private static AdminList createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        AdminList al = new AdminList();
        while (res.next())
        {
            Admin a = new Admin(res);
            al.add(a);
        }
        sql.close();
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
        
        return al;
    }
    
    //-------------------------------------------------------------------------

    /** constructors */
    private AdminList() {
        super();
    }

    //-------------------------------------------------------------------------
    
    /**
     * A utility method to return the BB user pk from an Admin object at the 
     * specified index (0-based) in this list.
     * @param idx The index of the Admin object in the list
     * @return The BB user pk associated with the Admin object at that index
     */
    public int getAdminPkAt(int idx)
    {
        Admin a = (Admin)this.get(idx);
        return a.getAdminUserPk();
    }
    
    /**
     * A utility methods that checks this list for any Admin object that grants
     * access for the specified user.
     * @param userPk the BB pk of the user to check for
     * @return true if there's an Admin object in this that grants admistrative privileges for 
     * the indicated user. false otherwise.
     */
    public boolean hasUserPk(int userPk)
    {
        for (int i = 0; i < this.size(); i++)
        {
            Admin a = (Admin)(this.get(i));
            if (userPk == a.getAdminUserPk())
            {
                return true;
            }
        }
        return false;
    }
    
    /**
     * A utility methods that checks this list for any Admin object that grants
     * access for the specified user.
     * @param userId the BB user_id (i.e. user name) of the user to check for
     * @return true if there's an Admin object in this that grants admistrative privileges for 
     * the indicated user. false otherwise.
     */
    public boolean hasUserId(String userId)
    {
        for (int i = 0; i < this.size(); i++)
        {
            Admin a = (Admin)(this.get(i));
            if (userId.equals(a.getUserId()))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * A utility methods that checks this list for any Admin object that grants
     * access for the specified user.
     * @param bbUser the BB User object
     * @return true if there's an Admin object in this that grants admistrative privileges for 
     * the indicated user. false otherwise.
     */
    public boolean hasBbUser(User bbUser)
    {
        int userPk = DataUtil.getPkFromBbId(bbUser.getId());
        return this.hasUserPk(userPk);
    }
}
