/*
 * FrameworkList.java
 *
 * Created on August 19, 2005, 9:27 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.util.ArrayList;
import wms.ss.db.Connect;
import blackboard.db.*;
import java.sql.*;
        
/**
 * This class manages a list of Framework objects. The constructor is private so creating
 * a FrameworkList is only done through the various loadByX static methods, which get all
 * the Frameworks that conform to the given criteria. See the loadByX methods for
 * more detail.
 * @author cwarren
 */
public class FrameworkList extends java.util.ArrayList {
    
    private static String dataSourceTable = "wms_ss_vw_framework_nd";
    
    /**
     * Generates a list of all Frameworks that the given user has user access (as
     * opposed to admin privileges) to, either directly via an Access or indirectly by
     * being a member of a course which has an Access.
     * @param userPk1 a BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a FrameworkList containing all the frameworks to which the user has access (i.e.
     * can sign up for)
     */
    public static FrameworkList loadByAccessUserId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText =
            "select ssframework.* " +
            " from wms_ss_vw_framework_nd ssframework, wms_ss_access ssaccess " +
            " where ssaccess.allowed_pk1="+Integer.toString(userPk1)+" " +
            " and ssaccess.deleted != 'Y' " +
            " and ssaccess.access_category='USER' " +
            " and ssframework.ss_framework_id = ssaccess.ss_framework_id " +
            " union " +
            " select ssframework.* " +
            " from wms_ss_vw_framework_nd ssframework, wms_ss_access ssaccess, course_users cu " +
            " where cu.users_pk1 = "+Integer.toString(userPk1)+" " +
            " and ssaccess.deleted != 'Y' " +
            " and ssaccess.allowed_pk1=cu.crsmain_pk1 " +
            " and cu.row_status=0 " +
            " and ssaccess.access_category='COURSE' " +
            " and ssframework.ss_framework_id = ssaccess.ss_framework_id " +
            " order by 7"; // ideally this'd be a column name, but oracle compains in that case. So, it's a column number for the framework name column
                
        return FrameworkList.createFromSql(sqlText);
    }

    /**
     * Generates a list of all the Frameworks a given user owns (i.e. was the creator)
     * @param userPk1 a BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a FrameworkList containing all the frameworks which the user owns
     */
    public static FrameworkList loadByOwnerUserId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return FrameworkList.createFromSql("SELECT * FROM "+dataSourceTable+
                " WHERE owner_user_pk1="+Integer.toString(userPk1) + " ORDER BY name");
    }

    /**
     * Generates a list of all Frameworks for which the given user has admin privileges (as
     * opposed to access), either directly because they own it or indirectly via an Admin.
     * @param userPk1 a BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a FrameworkList containing all the frameworks for which the given user has admin privileges
     */
    public static FrameworkList loadByAdminUserId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText =
            "select ssframework.* " +
            " from wms_ss_vw_framework_nd ssframework, wms_ss_vw_admin_nd ssadmin " +
            " where ssadmin.admin_user_pk="+Integer.toString(userPk1)+" " +
//            " and ssadmin.deleted != 'Y' " +
            " and ssframework.ss_framework_id = ssadmin.ss_framework_id " +
            " union " +
            " select ssframework.* " +
            " from wms_ss_vw_framework_nd ssframework " +
            " where ssframework.owner_user_pk1 = "+Integer.toString(userPk1)+" " +
            " order by 7"; // ideally this'd be a column name, but oracle compains in that case. So, it's a column number for the framework name column
                
        return FrameworkList.createFromSql(sqlText);
    }

    /**
     * Generates a list of all the Frameworks in a given SheetGroup
     * @param groupId the ID of a SheetGroup
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a FrameworkList containing all the frameworks in the given SheetGroup
     */
    public static FrameworkList loadByGroupId (int groupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return FrameworkList.createFromSql("SELECT * FROM "+dataSourceTable+
                " WHERE ss_group_id="+Integer.toString(groupId) + " ORDER BY name");
    }

    /**
     * Generates a list of all the Frameworks to which members of the specified course
     * have access.
     * @param coursePk a BB course PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a FrameworkList containing all the frameworks which have an Access for the given
     * course
     */
    public static FrameworkList loadByCourseId (int coursePk)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "SELECT ssframework.* " +
            "FROM wms_ss_vw_framework_nd ssframework, wms_ss_access ssaccess "+
            "WHERE ssaccess.ss_framework_id = ssframework.ss_framework_id "+
            " AND ssaccess.deleted != 'Y' " +
            " AND ssaccess.access_category = 'COURSE' "+
            " AND ssaccess.allowed_pk1 = " + Integer.toString(coursePk)
             + " ORDER BY name";

        return FrameworkList.createFromSql(sqlText);
    }

    /**
     * Generates a list of all the Frameworks to which members of the specified course
     * have access and the indicated user either owns the framework or has admin privs to it.
     * @param coursePk a BB course PK
     * @param userPk1 a BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a FrameworkList containing all the Frameworks to which members of the specified course
     * have access and the indicated user either owns the framework or has admin privs to it
     */
    public static FrameworkList loadByCourseIdAdminUserId (int coursePk, int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {

         String sqlText =
            "select ssframework.* " +
            " from wms_ss_vw_framework_nd ssframework, wms_ss_vw_admin_nd ssadmin, wms_ss_access ssaccess " +
            " where ssadmin.admin_user_pk="+Integer.toString(userPk1)+" " +
            " and ssframework.ss_framework_id = ssadmin.ss_framework_id " +
            " AND ssaccess.ss_framework_id = ssframework.ss_framework_id "+                 
            " AND ssaccess.deleted != 'Y' " +
            " AND ssaccess.access_category = 'COURSE' "+
            " AND ssaccess.allowed_pk1 = " + Integer.toString(coursePk) +
            " union " +
            " select ssframework.* " +
            " from wms_ss_vw_framework_nd ssframework, wms_ss_access ssaccess " +
            " where ssframework.owner_user_pk1 = "+Integer.toString(userPk1)+" " +
            " AND ssaccess.ss_framework_id = ssframework.ss_framework_id "+                 
            " AND ssaccess.deleted != 'Y' " +
            " AND ssaccess.access_category = 'COURSE' "+
            " AND ssaccess.allowed_pk1 = " + Integer.toString(coursePk) +
            " order by 7"; // ideally this'd be a column name, but oracle compains in that case. So, it's a column number for the framework name column
                
       return FrameworkList.createFromSql(sqlText);
    }

    /** private static methods */
    private static FrameworkList createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        FrameworkList fl = new FrameworkList();
        while (res.next())
        {
            Framework f = new Framework(res);
            fl.add(f);
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
        
        return fl;
    }

    
    /** object methods */

    /** constructors */
    private FrameworkList() {
        super();
    }
    
}
